"""Dumping for auxilliary types that are commonly used."""

from uuid import UUID
from pathlib import (
    Path,
    PosixPath,
    PurePath,
    PurePosixPath,
    PureWindowsPath,
    WindowsPath,
)

from pydantic_yaml._yaml import yaml


def _repr_path(dumper, node):
    return dumper.represent_str(str(node))


for _typ in [
    Path,
    PosixPath,
    PurePath,
    PurePosixPath,
    PureWindowsPath,
    WindowsPath,
]:
    yaml.add_representer(_typ, _repr_path)


# Fix dumping of UUID objects.


def _repr_uuid(dumper, node):
    return dumper.represent_str(str(node))


yaml.add_representer(UUID, _repr_uuid)


# Fix loading of Unicode strings from Python 2.


def _py2_str(loader, node):
    """Workaround to parsing YAML files generated by Python 2"""
    return node.value


yaml.SafeLoader.add_constructor("tag:yaml.org,2002:python/unicode", _py2_str)



from __future__ import annotations

from pathlib import Path
from typing import Callable, Optional, Type, Any, Union

from pydantic.main import BaseModel
from pydantic.error_wrappers import ErrorWrapper, ValidationError
from pydantic.types import StrBytes
from pydantic.parse import Protocol
from pydantic.utils import ROOT_KEY

from pydantic_yaml._yaml import yaml

__all__ = ["YamlModel"]

try:
    from typing import Literal

    ExtendedProto = Union[Protocol, Literal["yaml"]]
except ImportError:
    # I think this would happen with Python < 3.8
    ExtendedProto = Union[Protocol, str]


class YamlModel(BaseModel):
    """YAML-aware Pydantic model base class."""

    def yaml(
        self,
        *,
        include=None,
        exclude=None,
        by_alias: bool = False,
        skip_defaults: bool = None,
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
        encoder: Optional[Callable[[Any], Any]] = None,
        **dumps_kwargs: Any,
    ) -> str:
        """Generates a YAML representation of the model.
        
        Note that JSON is a subset of the YAMl spec, however this
        generates a multi-line, more-human-readable representation.
        """
        data = self.dict(
            include=include,
            exclude=exclude,
            by_alias=by_alias,
            exclude_unset=exclude_unset,
            exclude_defaults=exclude_defaults,
            exclude_none=exclude_none,
        )
        res = yaml.safe_dump(data)
        return res

    @classmethod
    def parse_raw(
        cls: Type[YamlModel],
        b: StrBytes,
        *,
        content_type: str = None,
        encoding: str = None,
        proto: ExtendedProto = None,
        allow_pickle: bool = False,
    ) -> YamlModel:
        if content_type is None:
            assume_yaml = False
        else:
            assume_yaml = ("yaml" in content_type) or ("yml" in content_type)

        if (proto == "yaml") or assume_yaml:
            try:
                obj = yaml.safe_load(b)
            except Exception as e:
                raise ValidationError([ErrorWrapper(e, loc=ROOT_KEY)], cls)
            return cls.parse_obj(obj)
        else:
            return super().parse_raw(
                b,
                content_type=content_type,
                encoding=encoding,
                proto=proto,
                allow_pickle=allow_pickle,
            )

    @classmethod
    def parse_file(
        cls: YamlModel,
        path: Union[str, Path],
        *,
        content_type: str = None,
        encoding: str = "utf-8",
        proto: ExtendedProto = None,
        allow_pickle: bool = False,
    ) -> YamlModel:
        if encoding is None:
            assume_yaml = False
        else:
            assume_yaml = ("yaml" in content_type) or ("yml" in content_type)

        if (proto == "yaml") or assume_yaml:
            content_type = content_type or "application/yaml"
            path = Path(path)
            b = path.read_bytes()
            return cls.parse_raw(
                b,
                content_type=content_type,
                encoding=encoding,
                proto="yaml",
                allow_pickle=allow_pickle,
            )
        else:
            return super().parse_file(
                path,
                content_type=content_type,
                encoding=encoding,
                proto=proto,
                allow_pickle=allow_pickle,
            )



"""YAML-enabled Pydantic models."""
from pathlib import Path

__all__ = ["__version__", "YamlEnum", "YamlModel"]

from . import aux_types  # noqa
from .enums import YamlEnum
from .models import YamlModel
from .version import __version__



from enum import Enum
from pydantic_yaml._yaml import yaml

__all__ = ["YamlEnum"]


def _repr_enum(dumper: yaml.Dumper, data: Enum):
    """Represent enums without additional hastles."""
    return dumper.represent_str(data.value)


class YamlEnum(Enum):
    """Enumeration that serializes as the proper underlying object type.

    You can use this instead of `enum.Enum`, for example:

        class MyEnum(str, YamlEnum):
            val1 = "Value 1"
            val2 = "Value 2"

    Note
    ----
    This is actually a lie, it only supports `str` enums for now. Oops.
    """

    def __init_subclass__(cls):
        yaml.add_representer(cls, _repr_enum)

    def __repr__(self) -> str:
        return repr(self.value)

    def __str__(self) -> str:
        return str(self.value)



# flake8: noqa

try:
    import ruamel.yaml as yaml

    if yaml.__version__ < "0.15.0":
        __yaml_lib__ = "ruamel-old"
    else:
        __yaml_lib__ = "ruamel-new"
except ImportError:
    try:
        import yaml

        __yaml_lib__ = "pyyaml"
    except ImportError:
        raise ImportError(
            "Could not import ruamel.yaml or pyyaml, "
            "please install at least one of them."
        )

__all__ = ["yaml", "__yaml_lib__"]



import importlib.util
from pathlib import Path

from setuptools import setup

description = "Adds some YAML functionality to the excellent `pydantic` library."

with (Path(__file__).parent / "README.md").open() as f:
    long_description = f.read()

# Get version
spec = importlib.util.spec_from_file_location(
    "pydantic_yaml.version", str(Path(__file__).parent / "pydantic_yaml/version.py")
)
vmodule = importlib.util.module_from_spec(spec)
spec.loader.exec_module(vmodule)
version = vmodule.__version__

# Run setup
setup(
    name="pydantic_yaml",
    version=version,
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Anatoly Makarevich",
    author_email="anatoly_mak@yahoo.com",
    url="https://github.com/NowanIlfideme/pydantic-yaml",
    license="MIT",
    packages=["pydantic_yaml"],
    python_requires=">=3.6",
    zip_safe=False,
    install_requires=["pydantic"],
    extras_require={
        "pyyaml": ["pyyaml"],
        "ruamel": ["ruamel.yaml>=0.15"],  # Using new API starting 0.15
        "ruamel-old": ["ruamel.yaml<0.15"],  # In-place repalcement
        "development": ["black", "flake8", "bump2version"],
    },
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Internet",
    ],
)
